"""Initial table creation

Peek Plugin Database Migration Script

Revision ID: d94277dd8aca
Revises: 
Create Date: 2017-06-25 14:53:23.543928

"""

# revision identifiers, used by Alembic.
revision = 'd94277dd8aca'
down_revision = None
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('DeviceInfo',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('description', sa.String(length=100), nullable=False),
    sa.Column('deviceId', sa.String(length=50), nullable=False),
    sa.Column('deviceType', sa.String(length=20), nullable=False),
    sa.Column('deviceToken', sa.String(length=50), nullable=False),
    sa.Column('appVersion', sa.String(length=15), nullable=False),
    sa.Column('updateVersion', sa.String(length=15), nullable=True),
    sa.Column('lastOnline', sa.DateTime(), nullable=True),
    sa.Column('lastUpdateCheck', sa.DateTime(), nullable=True),
    sa.Column('createdDate', sa.DateTime(), nullable=False),
    sa.Column('isOnline', sa.Boolean(), server_default='0', nullable=False),
    sa.Column('isEnrolled', sa.Boolean(), server_default='0', nullable=False),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('description'),
    sa.UniqueConstraint('deviceId'),
    sa.UniqueConstraint('deviceToken'),
    schema='core_device'
    )
    op.create_table('DeviceUpdate',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('deviceType', sa.String(length=20), nullable=False),
    sa.Column('description', sa.String(), nullable=False),
    sa.Column('buildDate', sa.DateTime(), nullable=False),
    sa.Column('appVersion', sa.String(length=15), nullable=False),
    sa.Column('updateVersion', sa.String(length=15), nullable=False),
    sa.Column('filePath', sa.String(length=150), nullable=False),
    sa.Column('urlPath', sa.String(length=150), nullable=False),
    sa.Column('isEnabled', sa.Boolean(), server_default='0', nullable=False),
    sa.PrimaryKeyConstraint('id'),
    schema='core_device'
    )
    op.create_index('idx_DeviceUpdate_Version', 'DeviceUpdate', ['deviceType', 'appVersion', 'updateVersion'], unique=True, schema='core_device')
    op.create_table('Setting',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(), nullable=True),
    sa.PrimaryKeyConstraint('id'),
    schema='core_device'
    )
    op.create_table('SettingProperty',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('settingId', sa.Integer(), nullable=False),
    sa.Column('key', sa.String(length=50), nullable=False),
    sa.Column('type', sa.String(length=16), nullable=True),
    sa.Column('int_value', sa.Integer(), nullable=True),
    sa.Column('char_value', sa.String(), nullable=True),
    sa.Column('boolean_value', sa.Boolean(), nullable=True),
    sa.ForeignKeyConstraint(['settingId'], ['core_device.Setting.id'], ),
    sa.PrimaryKeyConstraint('id'),
    schema='core_device'
    )
    op.create_index('idx_SettingProperty_settingId', 'SettingProperty', ['settingId'], unique=False, schema='core_device')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index('idx_SettingProperty_settingId', table_name='SettingProperty', schema='core_device')
    op.drop_table('SettingProperty', schema='core_device')
    op.drop_table('Setting', schema='core_device')
    op.drop_index('idx_DeviceUpdate_Version', table_name='DeviceUpdate', schema='core_device')
    op.drop_table('DeviceUpdate', schema='core_device')
    op.drop_table('DeviceInfo', schema='core_device')
    # ### end Alembic commands ###


import logging
from pathlib import Path

from twisted.internet import defer
from twisted.internet.defer import Deferred, inlineCallbacks

from peek_core_device._private.server.controller.NotifierController import \
    NotifierController
from peek_core_device._private.server.controller.UpdateController import \
    UpdateController
from peek_core_device._private.server.controller.EnrollmentController import \
    EnrollmentController
from peek_core_device._private.server.controller.OnlineController import OnlineController
from vortex.TupleAction import TupleActionABC
from vortex.handler.TupleActionProcessor import TupleActionProcessorDelegateABC
from vortex.handler.TupleDataObservableHandler import TupleDataObservableHandler

logger = logging.getLogger(__name__)


class MainController(TupleActionProcessorDelegateABC):
    def __init__(self, dbSessionCreator, notifierController: NotifierController,
                 deviceUpdateFilePath: Path):
        self._dbSessionCreator = dbSessionCreator
        self._notifierController = notifierController

        self._enrollmentController = EnrollmentController(
            dbSessionCreator, notifierController
        )

        self._onlineController = OnlineController(
            dbSessionCreator, notifierController
        )

        self._updateController = UpdateController(
            dbSessionCreator, notifierController, deviceUpdateFilePath
        )

    @property
    def deviceUpdateController(self):
        return self._updateController

    def shutdown(self):
        self._enrollmentController.shutdown()
        self._onlineController.shutdown()
        self._updateController.shutdown()

    @inlineCallbacks
    def processTupleAction(self, tupleAction: TupleActionABC) -> Deferred:
        result = yield self._enrollmentController.processTupleAction(tupleAction)
        if result is not None:
            defer.returnValue(result)

        result = yield self._onlineController.processTupleAction(tupleAction)
        if result is not None:
            defer.returnValue(result)

        result = yield self._updateController.processTupleAction(tupleAction)
        if result is not None:
            defer.returnValue(result)

        raise NotImplementedError(tupleAction.tupleName())



from vortex.handler.TupleActionProcessor import TupleActionProcessor

from peek_core_device._private.PluginNames import deviceFilt
from peek_core_device._private.PluginNames import deviceActionProcessorName
from .controller.MainController import MainController


def makeTupleActionProcessorHandler(mainController: MainController):
    processor = TupleActionProcessor(
        tupleActionProcessorName=deviceActionProcessorName,
        additionalFilt=deviceFilt,
        defaultDelegate=mainController)
    return processor



from peek_core_device._private.PluginNames import deviceFilt
from peek_core_device._private.PluginNames import deviceObservableName
from peek_plugin_base.PeekVortexUtil import peekServerName
from vortex.handler.TupleDataObservableProxyHandler import TupleDataObservableProxyHandler


def makeDeviceTupleDataObservableProxy():

    proxy =  TupleDataObservableProxyHandler(observableName=deviceObservableName,
                                           proxyToVortexName=peekServerName,
                                           additionalFilt=deviceFilt)

    return proxy



import logging

from sqlalchemy import Column
from sqlalchemy.sql.sqltypes import Boolean, Integer, String, DateTime

from .DeclarativeBase import DeclarativeBase
from vortex.Tuple import Tuple, addTupleType

from peek_core_device._private.PluginNames import deviceTuplePrefix

logger = logging.getLogger(__name__)


@addTupleType
class DeviceInfoTuple(Tuple, DeclarativeBase):
    """ DeviceInfoTuple

    This table stores information about devices.

    """
    __tablename__ = 'DeviceInfo'
    __tupleType__ = deviceTuplePrefix + 'DeviceInfoTuple'

    TYPE_MOBILE_IOS = "mobile-ios"
    TYPE_MOBILE_ANDROUD = "mobile-android"
    TYPE_MOBILE_WEB = "mobile-web"
    TYPE_DESKTOP_WEB = "desktop-web"
    TYPE_DESKTOP_WINDOWS = "desktop-windows"
    TYPE_DESKTOP_MACOS = "desktop-macos"

    id = Column(Integer, primary_key=True)
    description = Column(String(100), nullable=False, unique=True)
    deviceId = Column(String(50), nullable=False, unique=True)
    deviceType = Column(String(20), nullable=False)
    deviceToken = Column(String(50), nullable=False, unique=True)
    appVersion = Column(String(15), nullable=False)
    updateVersion = Column(String(15))  # Null means it hasn't updated
    lastOnline = Column(DateTime(True))
    lastUpdateCheck = Column(DateTime(True))
    createdDate = Column(DateTime(True), nullable=False)
    isOnline = Column(Boolean, nullable=False, server_default='0')
    isEnrolled = Column(Boolean, nullable=False, server_default='0')



import logging
from typing import Union

from peek_core_device._private.storage.Setting import OFFICE_ENROLLMENT_ENABLED, \
    FIELD_ENROLLMENT_ENABLED, globalSetting
from peek_core_device._private.tuples.ClientSettingsTuple import ClientSettingsTuple
from twisted.internet.defer import Deferred
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

logger = logging.getLogger(__name__)


class ClientSettingsTupleProvider(TuplesProviderABC):
    def __init__(self, ormSessionCreator):
        self._ormSessionCreator = ormSessionCreator

    @deferToThreadWrapWithLogger(logger)
    def makeVortexMsg(self, filt: dict,
                      tupleSelector: TupleSelector) -> Union[Deferred, bytes]:

        ormSession = self._ormSessionCreator()
        try:
            clientSetting = ClientSettingsTuple()

            clientSetting.fieldEnrollmentEnabled = globalSetting(ormSession,
                                                                 FIELD_ENROLLMENT_ENABLED)

            clientSetting.officeEnrollmentEnabled = globalSetting(ormSession,
                                                                  OFFICE_ENROLLMENT_ENABLED)

            # Create the vortex message
            return Payload(filt,
                           tuples=[clientSetting]).makePayloadEnvelope().toVortexMsg()

        finally:
            ormSession.close()



from peek_core_device._private.PluginNames import deviceTuplePrefix
from vortex.Tuple import addTupleType, TupleField
from vortex.TupleAction import TupleActionABC


@addTupleType
class UpdateEnrollmentAction(TupleActionABC):
    """ Authorise Enrolment Action

    This action authorises a device to enroll in this peek environment.

    """
    __tupleType__ = deviceTuplePrefix + 'UpdateEnrollmentAction'

    #:  The device info id to authorise
    deviceInfoId: int = TupleField()
    unenroll:bool = TupleField(False)
    remove:bool = TupleField(False)



from peek_core_device._private.storage.DeviceUpdateTuple import DeviceUpdateTuple
from vortex.Tuple import Tuple, addTupleType, TupleField

from peek_core_device._private.PluginNames import deviceTuplePrefix
from vortex.TupleAction import TupleActionABC


@addTupleType
class CreateDeviceUpdateAction(TupleActionABC):
    """ Create Update Action

    Create a new device update entry, this will be accompanied by an upload.

    """
    __tupleType__ = deviceTuplePrefix + 'CreateDeviceUpdateAction'

    #:  Description of date1
    newUpdate:DeviceUpdateTuple = TupleField()


import logging
import zipfile
from datetime import datetime
from pathlib import Path
from typing import List

import pytz
import shutil
from twisted.internet.defer import Deferred, inlineCallbacks, returnValue

from peek_core_device._private.server.controller.NotifierController import \
    NotifierController
from peek_core_device._private.storage.DeviceInfoTuple import DeviceInfoTuple
from peek_core_device._private.storage.DeviceUpdateTuple import DeviceUpdateTuple
from peek_core_device._private.tuples.AlterDeviceUpdateAction import \
    AlterDeviceUpdateAction
from peek_core_device._private.tuples.CreateDeviceUpdateAction import \
    CreateDeviceUpdateAction
from peek_core_device._private.tuples.UpdateAppliedAction import UpdateAppliedAction
from txhttputil.site.SpooledNamedTemporaryFile import SpooledNamedTemporaryFile
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Tuple import Tuple
from vortex.TupleAction import TupleActionABC
from vortex.handler.TupleDataObservableHandler import TupleDataObservableHandler

logger = logging.getLogger(__name__)


class UpdateController:
    def __init__(self, dbSessionCreator,
                 notifierController: NotifierController,
                 deviceUpdateFilePath: Path):
        self._dbSessionCreator = dbSessionCreator
        self._notifierController = notifierController
        self._deviceUpdateFilePath = deviceUpdateFilePath

    def shutdown(self):
        pass

    def processTupleAction(self, tupleAction: TupleActionABC) -> List[Tuple]:

        if isinstance(tupleAction, AlterDeviceUpdateAction):
            return self._processAdminAlter(tupleAction)

        if isinstance(tupleAction, UpdateAppliedAction):
            return self._processDeviceUpdated(tupleAction)

    @deferToThreadWrapWithLogger(logger)
    def _processAdminAlter(self, action: AlterDeviceUpdateAction) -> List[Tuple]:
        """ Process Admin Update

        :rtype: Deferred
        """
        session = self._dbSessionCreator()
        try:
            deviceUpdate = (
                session.query(DeviceUpdateTuple)
                    .filter(DeviceUpdateTuple.id == action.updateId)
                    .one()
            )

            deviceType = deviceUpdate.deviceType

            if action.remove:
                session.delete(deviceUpdate)
            else:
                deviceUpdate.isEnabled = action.isEnabled

            session.commit()

            self._notifierController.notifyDeviceUpdate(deviceType=deviceType)

            return []

        finally:
            # Always close the session after we create it
            session.close()

    @inlineCallbacks
    def processCreateUpdateUpload(self, namedTempFile: SpooledNamedTemporaryFile,
                                  action: CreateDeviceUpdateAction):
        """ Process Create Update Upload

        Unlike the other action processes in the controllers, this method is called by
        a http resource upload.
        """

        if not zipfile.is_zipfile(namedTempFile.name):
            raise Exception("Uploaded archive is not a zip file")

        # Create the file name
        filePath = '%s/%s.zip' % (
            action.newUpdate.deviceType, action.newUpdate.updateVersion)
        action.newUpdate.filePath = filePath
        action.newUpdate.urlPath = filePath.replace(r'\\', r'/')
        action.newUpdate.fileSize = Path(namedTempFile.name).stat().st_size

        # Create the database object, If that fails from some integrity problem
        # Then the file will delete it's self still
        deviceUpdateTuple = yield self._createUpdateOrmObj(action.newUpdate)

        absFilePath = self._deviceUpdateFilePath / filePath
        absFilePath.parent.mkdir(parents=True, exist_ok=True)

        shutil.move(namedTempFile.name, str(absFilePath))
        namedTempFile.delete = False

        self._notifierController.notifyDeviceUpdate(
            deviceType=deviceUpdateTuple.deviceType
        )

        returnValue("%s:%s Created Successfully"
                    % (deviceUpdateTuple.deviceType, deviceUpdateTuple.updateVersion))

    @deferToThreadWrapWithLogger(logger)
    def _createUpdateOrmObj(self, newUpdate: DeviceUpdateTuple) -> DeviceUpdateTuple:
        """ Process Device Enrolment

        :rtype: Deferred
        """
        ormSession = self._dbSessionCreator()
        try:
            newUpdate.buildDate = datetime.now(pytz.utc)
            newUpdate.isEnabled = False
            ormSession.add(newUpdate)
            ormSession.commit()

            ormSession.refresh(newUpdate)
            ormSession.expunge_all()
            return newUpdate

        finally:
            ormSession.close()



    @deferToThreadWrapWithLogger(logger)
    def _processDeviceUpdated(self, action: UpdateAppliedAction) -> List[Tuple]:
        """ Process Device Updated

        This action is sent when the device has applied an update, or attempted to.

        :rtype: Deferred
        """
        session = self._dbSessionCreator()
        try:
            deviceInfo = (
                session.query(DeviceInfoTuple)
                    .filter(DeviceInfoTuple.deviceId == action.deviceId)
                    .one()
            )

            deviceId = deviceInfo.deviceId

            if action.appVersion is not None:
                deviceInfo.appVersion = action.appVersion

            if action.updateVersion is not None:
                deviceInfo.updateVersion = action.updateVersion

            session.commit()

            self._notifierController.notifyDeviceInfo(deviceId=deviceId)

            return []

        finally:
            session.close()


from typing import Optional

from peek_core_device._private.PluginNames import deviceTuplePrefix
from vortex.Tuple import addTupleType, TupleField
from vortex.TupleAction import TupleActionABC


# I'm using the word Alter here, because UpdateUpdate is confusing.
@addTupleType
class AlterDeviceUpdateAction(TupleActionABC):
    """ Alter Device Update Tuple

    This action tuple applies changes to the Update from the admin UI.

    """
    __tupleType__ = deviceTuplePrefix + 'AlterDeviceUpdateAction'

    #:  the ID of the DeviceUpdateTuple
    updateId: int = TupleField()

    #: Set the enabled property of the update
    isEnabled: Optional[bool] = TupleField()

    #: Delete the update from the database
    remove: bool = TupleField(False)


from typing import Type

from peek_plugin_base.client.PluginClientEntryHookABC import PluginClientEntryHookABC
from peek_plugin_base.server.PluginServerEntryHookABC import PluginServerEntryHookABC

__version__ = '2.1.2'


def peekServerEntryHook() -> Type[PluginServerEntryHookABC]:
    from ._private.server.ServerEntryHook import ServerEntryHook
    return ServerEntryHook


def peekClientEntryHook() -> Type[PluginClientEntryHookABC]:
    from ._private.client.ClientEntryHook import ClientEntryHook
    return ClientEntryHook



from peek_plugin_base.PeekVortexUtil import peekServerName
from peek_core_device._private.PluginNames import deviceFilt
from peek_core_device._private.PluginNames import deviceActionProcessorName
from vortex.handler.TupleActionProcessorProxy import TupleActionProcessorProxy


def makeTupleActionProcessorProxy():
    return TupleActionProcessorProxy(
                tupleActionProcessorName=deviceActionProcessorName,
                proxyToVortexName=peekServerName,
                additionalFilt=deviceFilt)



from txhttputil.util.ModuleUtil import filterModules


def loadPublicTuples():
    """ Load Public Tuples

    In this method, we load the public tuples.
    This registers them so the Vortex can reconstructed them from
    serialised data.

    """
    for mod in filterModules(__name__, __file__):
        __import__(mod, locals(), globals())




from vortex.Tuple import addTupleType, TupleField
from vortex.TupleAction import TupleActionABC

from peek_core_device._private.PluginNames import deviceTuplePrefix


@addTupleType
class UpdateDeviceOnlineAction(TupleActionABC):
    __tupleType__ = deviceTuplePrefix + "UpdateDeviceOnlineAction"

    deviceId: str = TupleField()
    isOnline: bool = TupleField()


""" 
 * view.common.uiobj.Style.py
 *
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by 
 *  Synerty Pty Ltd
 *
"""
import json
import logging

from twisted.web.server import NOT_DONE_YET

from peek_core_device._private.server.controller.UpdateController import \
    UpdateController
from peek_core_device._private.tuples.CreateDeviceUpdateAction import \
    CreateDeviceUpdateAction
from txhttputil.site.BasicResource import BasicResource
from vortex.Payload import Payload

logger = logging.getLogger(name=__name__)


class DeviceUpdateUploadResource(BasicResource):
    isLeaf = True
    useLargeRequest = True

    UPDATE_TYPE_PLATFORM = 0
    UPDATE_TYPE_PLUGIN = 1

    def __init__(self, controller: UpdateController):
        BasicResource.__init__(self)
        self._controller = controller

    def render_GET(self, request):
        raise Exception("Updates must be done via the UI")

    def render_POST(self, request):
        request.responseHeaders.setRawHeaders('content-type', ['text/plain'])
        logger.info("Received device update upload request")

        action: CreateDeviceUpdateAction = Payload().fromVortexMsg(
            request.args[b'payload'][0]
        ).tuples[0]

        def good(data):
            request.write(json.dumps({'message': str(data)}).encode())
            request.finish()
            logger.info("Finished creating device update %s" % data)

        def bad(failure):
            e = failure.value
            logger.exception(e)

            request.write(json.dumps(
                {'error': str(failure.value),
                 'stdout': e.stdout if hasattr(e, 'stdout') else None,
                 'stderr': e.stderr if hasattr(e, 'stderr') else None}).encode())

            request.finish()

        d = self._controller.processCreateUpdateUpload(
            request.content.namedTemporaryFile,
            action
        )
        d.addCallbacks(good, bad)

        def closedError(failure):
            logger.error("Got closedError %s" % failure)

        request.notifyFinish().addErrback(closedError)

        return NOT_DONE_YET



"""added fileSize to update tuple

Peek Plugin Database Migration Script

Revision ID: 5dd43ff7ea8e
Revises: d94277dd8aca
Create Date: 2017-06-29 10:08:20.714750

"""

# revision identifiers, used by Alembic.
revision = '5dd43ff7ea8e'
down_revision = 'd94277dd8aca'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('DeviceUpdate', sa.Column('fileSize', sa.Integer(), nullable=False), schema='core_device')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('DeviceUpdate', 'fileSize', schema='core_device')
    # ### end Alembic commands ###


import logging
from typing import Union

from twisted.internet.defer import Deferred

from peek_core_device._private.storage.DeviceInfoTuple import DeviceInfoTuple
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

logger = logging.getLogger(__name__)

class DeviceInfoTupleProvider(TuplesProviderABC):
    def __init__(self, ormSessionCreator):
        self._ormSessionCreator = ormSessionCreator

    @deferToThreadWrapWithLogger(logger)
    def makeVortexMsg(self, filt: dict,
                      tupleSelector: TupleSelector) -> Union[Deferred, bytes]:

        deviceId = tupleSelector.selector.get("deviceId")

        ormSession = self._ormSessionCreator()
        try:
            qry = ormSession.query(DeviceInfoTuple)

            if deviceId is not None:
                qry = qry.filter(DeviceInfoTuple.deviceId == deviceId)

            tuples = qry.all()

            # Create the vortex message
            return Payload(filt, tuples=tuples).makePayloadEnvelope().toVortexMsg()

        finally:
            ormSession.close()



from typing import Optional, List

from abc import ABCMeta, abstractmethod
from rx import Observable
from twisted.internet.defer import Deferred


class DeviceApiABC(metaclass=ABCMeta):
    """ Device API

    This is the public API for the part of the plugin that runs on the server service.

    """

    @abstractmethod
    def deviceDetails(self, deviceTokens: List[str]) -> Deferred:
        """ Device Details

        Retrieve the details for the devices with the device tokens provided.

        :param deviceTokens: A list of device token strings
        :return: A Deferred that will fire with :code:`List[DeviceDetailTuple]`

        """

    @abstractmethod
    def deviceDescription(self, deviceToken: str) -> Deferred:
        """ Device Description

        Retrieve the devices description

        :param deviceToken: The token for the device to retrieve the description for
        :return: A Deferred that will fire with :code:`Optional[str]`

        """

    @abstractmethod
    def deviceDescriptionBlocking(self, deviceToken: str) -> Optional[str]:
        """ Device Description

        Retrieve the devices description, this must be called from a thread.

        :param deviceToken: The token for the device to retrieve the description for
        :return: The device description or None

        """

    @abstractmethod
    def deviceOnlineStatus(self) -> Observable:
        """ Device Online Stauts

        Subscribe to device online status

        :return: An observable that fires when devices go online or offline.

        """



from vortex.Tuple import addTupleType, TupleField
from vortex.TupleAction import TupleActionABC

from peek_core_device._private.PluginNames import deviceTuplePrefix


@addTupleType
class EnrolDeviceAction(TupleActionABC):
    __tupleType__ = deviceTuplePrefix + "EnrolDeviceAction"

    description:str = TupleField()
    deviceId: str = TupleField()
    deviceType: str = TupleField()
    appVersion: str = TupleField()


from vortex.handler.TupleDataObservableHandler import TupleDataObservableHandler
from .SettingPropertyHandler import makeSettingPropertyHandler


def makeAdminBackendHandlers(tupleObservable: TupleDataObservableHandler,
                             dbSessionCreator):

    yield makeSettingPropertyHandler(tupleObservable, dbSessionCreator)
